@@ -7,7 +7,7 @@
  * @date   2006-09-13
  * @brief  LibRaw settings widgets
  *
- * @author Copyright (C) 2006-2012 by Gilles Caulier
+ * @author Copyright (C) 2006-2013 by Gilles Caulier
  *         <a href="mailto:caulier dot gilles at gmail dot com">caulier dot gilles at gmail dot com</a>
  * @author Copyright (C) 2006-2011 by Marcel Wiesweg
  *         <a href="mailto:marcel dot wiesweg at gmx dot de">marcel dot wiesweg at gmx dot de</a>
@@ -121,6 +121,26 @@ public:
         expoCorrectionHighlightLabel   = 0;
     }
 
+    /** Convert Exposure correction shift E.V value from GUI to Linear value needs by libraw decoder.
+     */
+    double shiftExpoFromEvToLinear(double ev) const
+    {
+        // From GUI : -2.0EV => 0.25
+        //            +3.0EV => 8.00  
+        return (1.55*ev + 3.35);
+    }
+
+    /** Convert Exposure correction shift Linear value from liraw decoder to E.V value needs by GUI.
+     */
+    double shiftExpoFromLinearToEv(double lin) const
+    {
+        // From GUI : 0.25 => -2.0EV
+        //            8.00 => +3.0EV  
+        return ((lin-3.35) / 1.55);
+    }
+
+public:
+
     QWidget*         demosaicingSettings;
     QWidget*         whiteBalanceSettings;
     QWidget*         correctionsSettings;
@@ -195,8 +215,8 @@ void DcrawSettingsWidget::setup(int advSettings)
     // ---------------------------------------------------------------
     // DEMOSAICING Settings panel
 
-    d->demosaicingSettings         = new QWidget(this);
-    QGridLayout* demosaicingLayout = new QGridLayout(d->demosaicingSettings);
+    d->demosaicingSettings               = new QWidget(this);
+    QGridLayout* const demosaicingLayout = new QGridLayout(d->demosaicingSettings);
 
     int line = 0;
 
@@ -233,13 +253,13 @@ void DcrawSettingsWidget::setup(int advSettings)
     demosaicingLayout->addWidget(d->fourColorCheckBox, line, 0, 1, line == 0 ? 2 : 3);
     line++;
 
-    KUrlLabel *dcrawVersion = new KUrlLabel("http://www.libraw.org", QString("libraw %1")
+    KUrlLabel* const dcrawVersion = new KUrlLabel("http://www.libraw.org", QString("libraw %1")
                                   .arg(KDcraw::librawVersion()), d->demosaicingSettings);
     dcrawVersion->setAlignment(Qt::AlignRight);
     dcrawVersion->setToolTip(i18nc("@info:tooltip", "Visit LibRaw project website"));
     demosaicingLayout->addWidget(dcrawVersion, 0, 2, 1, 1);
 
-    d->dontStretchPixelsCheckBox = new QCheckBox(i18nc("@option:check", "Do not stretch or rotate pixels"), d->demosaicingSettings);
+    d->dontStretchPixelsCheckBox  = new QCheckBox(i18nc("@option:check", "Do not stretch or rotate pixels"), d->demosaicingSettings);
     d->dontStretchPixelsCheckBox->setWhatsThis(i18nc("@info:whatsthis", 
                                 "<title>Do not stretch or rotate pixels</title>"
                                 "<para>For Fuji Super CCD cameras, show the image tilted 45 degrees. "
@@ -258,16 +278,31 @@ void DcrawSettingsWidget::setup(int advSettings)
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::PPG,      i18nc("@item:inlistbox Quality", "PPG"));
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::AHD,      i18nc("@item:inlistbox Quality", "AHD"));
 
-    // Extended demosaicing method
+    // Extended demosaicing method from GPL2 pack
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::DCB,      i18nc("@item:inlistbox Quality", "DCB"));
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::PL_AHD,   i18nc("@item:inlistbox Quality", "AHD v2"));
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::AFD,      i18nc("@item:inlistbox Quality", "AFD"));
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::VCD,      i18nc("@item:inlistbox Quality", "VCD"));
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::VCD_AHD,  i18nc("@item:inlistbox Quality", "VCD & AHD"));
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::LMMSE,    i18nc("@item:inlistbox Quality", "LMMSE"));
+    // Extended demosaicing method from GPL3 pack
     d->RAWQualityComboBox->insertItem(RawDecodingSettings::AMAZE,    i18nc("@item:inlistbox Quality", "AMaZE"));
 
+    // If Libraw do not support GPL2 pack, disable entries relevant.
+    if (!KDcraw::librawUseGPL2DemosaicPack())
+    {
+        for (int i=RawDecodingSettings::DCB ; i <=RawDecodingSettings::LMMSE ; ++i)
+            d->RAWQualityComboBox->combo()->setItemData(i, false, Qt::UserRole-1);
+    }
+
+    // If Libraw do not support GPL3 pack, disable entries relevant.
+    if (!KDcraw::librawUseGPL3DemosaicPack())
+    {
+        d->RAWQualityComboBox->combo()->setItemData(RawDecodingSettings::AMAZE, false, Qt::UserRole-1);
+    }
+
     d->RAWQualityComboBox->setDefaultIndex(RawDecodingSettings::BILINEAR);
+    d->RAWQualityComboBox->setCurrentIndex(RawDecodingSettings::BILINEAR);
     d->RAWQualityComboBox->setWhatsThis(i18nc("@info:whatsthis", "<title>Quality (interpolation)</title>"
                                 "<para>Select here the demosaicing method to use when decoding RAW "
                                 "images. A demosaicing algorithm is a digital image process used to "
@@ -278,7 +313,7 @@ void DcrawSettingsWidget::setup(int advSettings)
                                 "demosaicing. The following methods are available for demosaicing "
                                 "RAW images:</para>"
 
-                                        // Original dcraw demosaicing methods
+                                // Original dcraw demosaicing methods
 
                                 "<para><list><item><emphasis strong='true'>Bilinear</emphasis>: use "
                                 "high-speed but low-quality bilinear interpolation (default - for "
@@ -301,7 +336,7 @@ void DcrawSettingsWidget::setup(int advSettings)
                                 "direction of interpolation so as to maximize a homogeneity metric, "
                                 "thus typically minimizing color artifacts.</item>"
 
-                                        // Extended demosaicing method
+                                // Extended demosaicing method
 
                                 "<item><emphasis strong='true'>DCB</emphasis>: DCB interpolation from "
                                 "linuxphoto.org project.</item>"
@@ -325,7 +360,10 @@ void DcrawSettingsWidget::setup(int advSettings)
 
                                 "<item><emphasis strong='true'>AMaZE</emphasis>: Aliasing Minimization "
                                 "interpolation and Zipper Elimination to apply color aberration removal "
-                                "from RawTherapee project.</item></list></para>"));
+                                "from RawTherapee project.</item></list></para>"
+                                
+                                "<para>Note: some methods can be unavailable if RAW decoder have been built "
+                                "without extension packs.</para>"));
 
     demosaicingLayout->addWidget(d->RAWQualityLabel,    line, 0, 1, 1);
     demosaicingLayout->addWidget(d->RAWQualityComboBox, line, 1, 1, 2);
@@ -348,7 +386,7 @@ void DcrawSettingsWidget::setup(int advSettings)
     demosaicingLayout->addWidget(d->medianFilterPassesSpinBox, line, 1, 1, 2);
     line++;
 
-    d->refineInterpolationBox       = new QCheckBox(i18nc("@option:check", "Refine interpolation"), d->demosaicingSettings);
+    d->refineInterpolationBox = new QCheckBox(i18nc("@option:check", "Refine interpolation"), d->demosaicingSettings);
     d->refineInterpolationBox->setWhatsThis(i18nc("@info:whatsthis", "<title>Refine interpolation</title>"
                                 "<para>This setting is available only for few Quality options:</para>"
                                 "<para><list><item><emphasis strong='true'>DCB</emphasis>: turn on "
@@ -357,17 +395,25 @@ void DcrawSettingsWidget::setup(int advSettings)
                                 "enhanced effective color interpolation (EECI) refine to improve "
                                 "sharpness.</item></list></para>"));
     demosaicingLayout->addWidget(d->refineInterpolationBox, line, 0, 1, 2);
+         
+    // If Libraw do not support GPL2 pack, disable options relevant.
+    if (!KDcraw::librawUseGPL2DemosaicPack())
+    {
+        d->medianFilterPassesLabel->setEnabled(false);
+        d->medianFilterPassesSpinBox->setEnabled(false);
+        d->refineInterpolationBox->setEnabled(false);
+    }
 
     addItem(d->demosaicingSettings, SmallIcon("kdcraw"), i18nc("@label", "Demosaicing"), QString("demosaicing"), true);
 
     // ---------------------------------------------------------------
     // WHITE BALANCE Settings Panel
 
-    d->whiteBalanceSettings         = new QWidget(this);
-    QGridLayout* whiteBalanceLayout = new QGridLayout(d->whiteBalanceSettings);
+    d->whiteBalanceSettings               = new QWidget(this);
+    QGridLayout* const whiteBalanceLayout = new QGridLayout(d->whiteBalanceSettings);
 
-    d->whiteBalanceLabel            = new QLabel(i18nc("@label:listbox", "Method:"), d->whiteBalanceSettings);
-    d->whiteBalanceComboBox         = new RComboBox(d->whiteBalanceSettings);
+    d->whiteBalanceLabel    = new QLabel(i18nc("@label:listbox", "Method:"), d->whiteBalanceSettings);
+    d->whiteBalanceComboBox = new RComboBox(d->whiteBalanceSettings);
     d->whiteBalanceComboBox->insertItem(RawDecodingSettings::NONE,   i18nc("@item:inlistbox", "Default D65"));
     d->whiteBalanceComboBox->insertItem(RawDecodingSettings::CAMERA, i18nc("@item:inlistbox", "Camera"));
     d->whiteBalanceComboBox->insertItem(RawDecodingSettings::AUTO,   i18nc("@item:inlistbox set while balance automatically", "Automatic"));
@@ -428,27 +474,26 @@ void DcrawSettingsWidget::setup(int advSettings)
                                 "<para>Specify the reconstruct highlight level. Low values favor "
                                 "whites and high values favor colors.</para>"));
 
-    d->expoCorrectionBox = new QCheckBox(i18nc("@option:check", "Exposure Correction"), d->whiteBalanceSettings);
+    d->expoCorrectionBox = new QCheckBox(i18nc("@option:check", "Exposure Correction (E.V)"), d->whiteBalanceSettings);
     d->expoCorrectionBox->setWhatsThis(i18nc("@info:whatsthis", "<para>Turn on the exposure "
                                 "correction before interpolation.</para>"));
 
-    d->expoCorrectionShiftLabel   = new QLabel(i18nc("@label:slider", "Shift (linear):"), d->whiteBalanceSettings);
+    d->expoCorrectionShiftLabel   = new QLabel(i18nc("@label:slider", "Linear Shift:"), d->whiteBalanceSettings);
     d->expoCorrectionShiftSpinBox = new RDoubleNumInput(d->whiteBalanceSettings);
     d->expoCorrectionShiftSpinBox->setDecimals(2);
-    d->expoCorrectionShiftSpinBox->setRange(0.5, 3.0, 0.01);
-    d->expoCorrectionShiftSpinBox->setDefaultValue(1.0);
+    d->expoCorrectionShiftSpinBox->setRange(-2.0, 3.0, 0.01);
+    d->expoCorrectionShiftSpinBox->setDefaultValue(0.0);
     d->expoCorrectionShiftSpinBox->setWhatsThis(i18nc("@info:whatsthis", "<title>Shift</title>"
-                                "<para>Shift of exposure correction before interpolation in linear "
-                                "scale.</para>"));
+                                "<para>Linear Shift of exposure correction before interpolation in E.V</para>"));
 
-    d->expoCorrectionHighlightLabel   = new QLabel(i18nc("@label:slider", "Highlight (E.V):"), d->whiteBalanceSettings);
+    d->expoCorrectionHighlightLabel   = new QLabel(i18nc("@label:slider", "Highlight:"), d->whiteBalanceSettings);
     d->expoCorrectionHighlightSpinBox = new RDoubleNumInput(d->whiteBalanceSettings);
     d->expoCorrectionHighlightSpinBox->setDecimals(2);
     d->expoCorrectionHighlightSpinBox->setRange(0.0, 1.0, 0.01);
     d->expoCorrectionHighlightSpinBox->setDefaultValue(0.0);
     d->expoCorrectionHighlightSpinBox->setWhatsThis(i18nc("@info:whatsthis", "<title>Highlight</title>"
                                 "<para>Amount of highlight preservation for exposure correction "
-                                "before interpolation in E.V.</para>"));
+                                "before interpolation in E.V. Only take effect if Shift Correction is > 1.0 E.V</para>"));
 
     d->fixColorsHighlightsBox = new QCheckBox(i18nc("@option:check", "Correct false colors in highlights"), d->whiteBalanceSettings);
     d->fixColorsHighlightsBox->setWhatsThis(i18nc("@info:whatsthis", "<para>If enabled, images with "
@@ -537,8 +582,8 @@ void DcrawSettingsWidget::setup(int advSettings)
     // ---------------------------------------------------------------
     // CORRECTIONS Settings panel
 
-    d->correctionsSettings         = new QWidget(this);
-    QGridLayout* correctionsLayout = new QGridLayout(d->correctionsSettings);
+    d->correctionsSettings               = new QWidget(this);
+    QGridLayout* const correctionsLayout = new QGridLayout(d->correctionsSettings);
 
     d->noiseReductionLabel    = new QLabel(i18nc("@label:listbox", "Noise reduction:"), d->correctionsSettings);
     d->noiseReductionComboBox = new RComboBox(d->colormanSettings);
@@ -627,8 +672,8 @@ void DcrawSettingsWidget::setup(int advSettings)
     // ---------------------------------------------------------------
     // COLOR MANAGEMENT Settings panel
 
-    d->colormanSettings         = new QWidget(this);
-    QGridLayout* colormanLayout = new QGridLayout(d->colormanSettings);
+    d->colormanSettings               = new QWidget(this);
+    QGridLayout* const colormanLayout = new QGridLayout(d->colormanSettings);
 
     d->inputColorSpaceLabel     = new QLabel(i18nc("@label:listbox", "Camera Profile:"), d->colormanSettings);
     d->inputColorSpaceComboBox  = new RComboBox(d->colormanSettings);
@@ -738,6 +783,9 @@ void DcrawSettingsWidget::setup(int advSettings)
     connect(d->expoCorrectionBox, SIGNAL(toggled(bool)),
             this, SLOT(slotExposureCorrectionToggled(bool)));
 
+    connect(d->expoCorrectionShiftSpinBox, SIGNAL(valueChanged(double)),
+            this, SLOT(slotExpoCorrectionShiftChanged(double)));
+
     // Wrapper to emit signal when something is changed in settings.
 
     connect(d->inIccUrlEdit, SIGNAL(urlSelected(KUrl)),
@@ -818,9 +866,6 @@ void DcrawSettingsWidget::setup(int advSettings)
     connect(d->brightnessSpinBox, SIGNAL(valueChanged(double)),
             this, SIGNAL(signalSettingsChanged()));
 
-    connect(d->expoCorrectionShiftSpinBox, SIGNAL(valueChanged(double)),
-            this, SIGNAL(signalSettingsChanged()));
-
     connect(d->expoCorrectionHighlightSpinBox, SIGNAL(valueChanged(double)),
             this, SIGNAL(signalSettingsChanged()));
 }
@@ -833,9 +878,12 @@ DcrawSettingsWidget::~DcrawSettingsWidget()
 void DcrawSettingsWidget::updateMinimumWidth()
 {
     int width = 0;
+
     for (int i = 0; i < count(); i++)
+    {
         if (widget(i)->width() > width)
             width = widget(i)->width();
+    }
 
     setMinimumWidth(width);
 }
@@ -955,7 +1003,7 @@ void DcrawSettingsWidget::slotAutoCAToggled(bool b)
         d->caBlueMultSpinBox->setValue(0.0);
     }
 
-    bool mult = !b & d->autoCACorrectionBox->isEnabled();
+    bool mult = (!b) && (d->autoCACorrectionBox->isEnabled());
     d->caRedMultSpinBox->setEnabled(mult);
     d->caBlueMultSpinBox->setEnabled(mult);
     d->caRedMultLabel->setEnabled(mult);
@@ -970,6 +1018,17 @@ void DcrawSettingsWidget::slotExposureCorrectionToggled(bool b)
     d->expoCorrectionHighlightLabel->setEnabled(b);
     d->expoCorrectionHighlightSpinBox->setEnabled(b);
 
+    slotExpoCorrectionShiftChanged(d->expoCorrectionShiftSpinBox->value());
+}
+
+void DcrawSettingsWidget::slotExpoCorrectionShiftChanged(double ev)
+{
+    // Only enable Highligh exposure correction if Shift correction is >= 1.0, else this settings do not take effect.
+    bool b = (ev >= 1.0);
+
+    d->expoCorrectionHighlightLabel->setEnabled(b);
+    d->expoCorrectionHighlightSpinBox->setEnabled(b);
+    
     emit signalSettingsChanged();
 }
 
@@ -988,41 +1047,17 @@ void DcrawSettingsWidget::slotRAWQualityChanged(int quality)
     switch(quality)
     {
         case RawDecodingSettings::DCB:
-            d->medianFilterPassesLabel->setEnabled(true);
-            d->medianFilterPassesSpinBox->setEnabled(true);
-            d->refineInterpolationBox->setEnabled(true);
+        case RawDecodingSettings::VCD_AHD:
+            // These options can be only avaialble if Libraw use GPL2 pack.
+            d->medianFilterPassesLabel->setEnabled(KDcraw::librawUseGPL2DemosaicPack());
+            d->medianFilterPassesSpinBox->setEnabled(KDcraw::librawUseGPL2DemosaicPack());
+            d->refineInterpolationBox->setEnabled(KDcraw::librawUseGPL2DemosaicPack());
             break;
 
         case RawDecodingSettings::PL_AHD:
-            d->medianFilterPassesLabel->setEnabled(false);
-            d->medianFilterPassesSpinBox->setEnabled(false);
-            d->refineInterpolationBox->setEnabled(false);
-            break;
-
         case RawDecodingSettings::AFD:
-            d->medianFilterPassesLabel->setEnabled(false);
-            d->medianFilterPassesSpinBox->setEnabled(false);
-            d->refineInterpolationBox->setEnabled(false);
-            break;
-
         case RawDecodingSettings::VCD:
-            d->medianFilterPassesLabel->setEnabled(false);
-            d->medianFilterPassesSpinBox->setEnabled(false);
-            d->refineInterpolationBox->setEnabled(false);
-            break;
-
-        case RawDecodingSettings::VCD_AHD:
-            d->medianFilterPassesLabel->setEnabled(true);
-            d->medianFilterPassesSpinBox->setEnabled(true);
-            d->refineInterpolationBox->setEnabled(true);
-            break;
-
         case RawDecodingSettings::LMMSE:
-            d->medianFilterPassesLabel->setEnabled(false);
-            d->medianFilterPassesSpinBox->setEnabled(false);
-            d->refineInterpolationBox->setEnabled(false);
-            break;
-
         case RawDecodingSettings::AMAZE:
             d->medianFilterPassesLabel->setEnabled(false);
             d->medianFilterPassesSpinBox->setEnabled(false);
@@ -1035,6 +1070,7 @@ void DcrawSettingsWidget::slotRAWQualityChanged(int quality)
             d->refineInterpolationBox->setEnabled(false);
             break;
     }
+    
     emit signalSettingsChanged();
 }
 
@@ -1103,8 +1139,32 @@ void DcrawSettingsWidget::setSettings(const RawDecodingSettings& settings)
     d->whitePointSpinBox->setEnabled(settings.enableWhitePoint);
     d->whitePointSpinBox->setValue(settings.whitePoint);
 
-    d->RAWQualityComboBox->setCurrentIndex(settings.RAWQuality);
-    switch(settings.RAWQuality)
+    int q = settings.RAWQuality;
+    
+    // If Libraw do not support GPL2 pack, reset to BILINEAR.
+    if (!KDcraw::librawUseGPL2DemosaicPack())
+    {
+        for (int i=RawDecodingSettings::DCB ; i <=RawDecodingSettings::LMMSE ; ++i)
+        {
+            if (q == i)
+            {
+                q = RawDecodingSettings::BILINEAR;
+                kDebug() << "Libraw GPL2 pack not avaialble. Raw quality set to Bilinear";
+                break;
+            }
+        }
+    }
+
+    // If Libraw do not support GPL3 pack, reset to BILINEAR.
+    if (!KDcraw::librawUseGPL3DemosaicPack() && (q == RawDecodingSettings::AMAZE))
+    {
+        q = RawDecodingSettings::BILINEAR;
+        kDebug() << "Libraw GPL3 pack not avaialble. Raw quality set to Bilinear";
+    }
+    
+    d->RAWQualityComboBox->setCurrentIndex(q);
+
+    switch(q)
     {
         case RawDecodingSettings::DCB:
             d->medianFilterPassesSpinBox->setValue(settings.dcbIterations);
@@ -1119,7 +1179,8 @@ void DcrawSettingsWidget::setSettings(const RawDecodingSettings& settings)
             d->refineInterpolationBox->setChecked(false); // option not used.
             break;
     }
-    slotRAWQualityChanged(settings.RAWQuality);
+
+    slotRAWQualityChanged(q);
 
     d->inputColorSpaceComboBox->setCurrentIndex((int)settings.inputColorSpace);
     slotInputColorSpaceChanged((int)settings.inputColorSpace);
@@ -1139,7 +1200,7 @@ void DcrawSettingsWidget::setSettings(const RawDecodingSettings& settings)
 
     d->expoCorrectionBox->setChecked(settings.expoCorrection);
     slotExposureCorrectionToggled(settings.expoCorrection);
-    d->expoCorrectionShiftSpinBox->setValue(settings.expoCorrectionShift);
+    d->expoCorrectionShiftSpinBox->setValue(d->shiftExpoFromLinearToEv(settings.expoCorrectionShift));
     d->expoCorrectionHighlightSpinBox->setValue(settings.expoCorrectionHighlight);
 
     d->inIccUrlEdit->setUrl(KUrl(settings.inputProfile));
@@ -1227,6 +1288,7 @@ RawDecodingSettings DcrawSettingsWidget::settings() const
         {
             prm.NRThreshold     = d->NRSpinBox1->value();
             prm.NRChroThreshold = 0;
+            break;
         }
         default:    // IMPULSENR
         {
@@ -1241,7 +1303,7 @@ RawDecodingSettings DcrawSettingsWidget::settings() const
     prm.caMultiplier[1]         = d->caBlueMultSpinBox->value();
 
     prm.expoCorrection          = d->expoCorrectionBox->isChecked();
-    prm.expoCorrectionShift     = d->expoCorrectionShiftSpinBox->value();
+    prm.expoCorrectionShift     = d->shiftExpoFromEvToLinear(d->expoCorrectionShiftSpinBox->value());
     prm.expoCorrectionHighlight = d->expoCorrectionHighlightSpinBox->value();
 
     prm.inputColorSpace         = (RawDecodingSettings::InputColorSpace)(d->inputColorSpaceComboBox->currentIndex());
